/* ---------------------------------------------------------------- */
//
// AppDelegate.m
//
// Created by Christoph Birk on 2020-12-21
// Copyright (c) 2016 Christoph Birk. All rights reserved.
//
/* ---------------------------------------------------------------- */

#define DEBUG           1
#define DBE_DATAPATH    "dbe_dataPath"

/* ---------------------------------------------------------------- */

#import  "AppDelegate.h"
#import  "QltoolController.h"
#import  "FITS.h"
#import  "main.h" 
#import  "Tcpip.h" // NEW

/* ---------------------------------------------------------------- */

@implementation AppDelegate
{
  u_int counter;
  QltoolController *qlt_controller;
  TCPIP_Server *tcpipServer; // NEW
}

/* ---------------------------------------------------------------- */
#pragma mark "Class"
/* ---------------------------------------------------------------- */
//34567890123456789012345678901234567890


/* ---------------------------------------------------------------- */
#pragma mark "NSApp Delegate"
/* ---------------------------------------------------------------- */
//34567890123456789012345678901234567890

- (void)applicationDidFinishLaunching:(NSNotification*)note
{
  char buf[128];
  NSString *str;
#if (DEBUG > 0)
  fprintf(stderr,"%s(%p)\n",PREFUN,note);
#endif

  YbundleInfo(&p_exec,&p_build,&p_version,NULL,&r_path,&p_ident);

  // when the current user is no in the list (ie. if it's not 'birk')
  // we write the preferences to /Users/Shared/Library/Preferences
  // to allow different users to work with the same prefrences
  NSArray *users = [NSArray arrayWithObjects:@"birk",nil];
  if ([users indexOfObject:NSUserName()] == NSNotFound) {
    use_shared = YprefSetupShared();
  } else {
    use_shared = NO;
  }
  (void)YappSupportCreate(&s_path,use_shared);

  main_logger = [[Logger alloc] initWithBase:nil useShared:use_shared];
  // main_logger.mode = LOGGER_FLUSH;        // keep open, auto-flush
  str = [NSString stringWithFormat:@"user= %@ , sharedSetup= %d",NSUserName(),use_shared];
  [main_logger append:[str UTF8String]];
  
  self.dataPath = YprefGetString(@DBE_DATAPATH,NSHomeDirectory(),use_shared);
  
  qlt_controller = [[QltoolController alloc] initWithWindowNib:@"Qltool"];
  
  // NEW create the TCP/IP server that listens to commands.
  // Requires a delegate method 'tcpip_handler' (below).
  const int port = 50001;
  tcpipServer = [[TCPIP_Server alloc] initWithPort:port delegate:self];
  int err = [tcpipServer start];
  if (err) { NSString *s;
    sprintf(buf,"TCP/IP server failed, err=%d",err);
    s = [NSString stringWithFormat:@"%s\r%@",buf,[TCPIP_Server errorString:err]];
    [main_logger message:[s UTF8String] level:1 file:YES];
  } else {
    sprintf(buf,"listening to port %d\n",port);
    [main_logger append:buf];
  }

  // This is an example to show you how to write a client; ie. what kind
  // of code I'll run at the IFUM-GUI.
  // For manual testing of your server I recommend the 'nc' (netcat) utility
  // xterm> nc localhost 50001
  // then type commands (eg. version<return>)
#if 1 // NEW just for testing 
  dispatch_async(GLOBAL_QUEUE,^(void) { char buf[128],res[128];
    Ysleep(500);
    TCPIP_Connection *con = [[TCPIP_Connection alloc] initWithHost:"localhost" port:port online:YES];
    int err = [con open];
    if (!err) {
      err = [con request:"version\n" response:res max:sizeof(res)];
      [con close];
    }
    if (err) sprintf(buf,"TCP/IP error=%d",err);
    else     sprintf(buf,"version= %s",res);
    [Notice openNotice:buf title:p_exec];
    [con release];
  });
#endif
}

/* ---------------------------------------------------------------- */

- (BOOL)applicationShouldTerminateAfterLastWindowClosed:(NSApplication*)appl
{
#if (DEBUG > 0)
  fprintf(stderr,"%s(%p)\n",PREFUN,appl);
#endif
  return YES;
}

/* ---------------------------------------------------------------- */

- (NSApplicationTerminateReply)applicationShouldTerminate:(NSApplication*)appl
{
#if (DEBUG > 0)
  fprintf(stderr,"%s(%p)\n",PREFUN,appl);
#endif

  return NSTerminateNow;
}

/* ---------------------------------------------------------------- */

- (void)applicationWillTerminate:(NSNotification *)note
{
#if (DEBUG > 0)
  fprintf(stderr,"%s(%p)\n",PREFUN,note);
#endif

  [qlt_controller close];

  [main_logger append:PREFUN];
  [main_logger close];

  YprefPutString(@DBE_DATAPATH,_dataPath,use_shared);

  [[NSUserDefaults standardUserDefaults] synchronize];
}

/* ---------------------------------------------------------------- */
#pragma mark "Instance"
/* ---------------------------------------------------------------- */

- (void)dealloc
{
  [_dataPath release];
  [super dealloc];
}

/* ---------------------------------------------------------------- */

- (void)loadFITS:(NSURL*)url // NEW changed from NSArray to NSURL
{
  int n1,n2;
#if (DEBUG > 0)
  fprintf(stderr,"%s(%p)\n",PREFUN,url);
#endif

  [main_logger append:[[url path] UTF8String]];
  int *data = fits_load32([[url path] UTF8String],&n1,&n2,0);
  // NSString *file_map = @"/Users/birk/Desktop/xapb_Config30B_map_210112.fits";
  // data = fits_load32([file_map UTF8String], &n1, &n2, 0);
  if (data) { int *dbuf=data;
    [qlt_controller reset:0 width:n1 height:n2 title:[url lastPathComponent]];
    for (int i=0; i<n2; i++,dbuf+=n1) [qlt_controller updateData:dbuf line:i];
    [qlt_controller redrawWithScaling:YES];
    free((void*)data);
  } else {
    NSString *string = [NSString stringWithFormat:@"FITS: illegal format\r%@",[url path]];
    [Notice openNotice:[string UTF8String] title:p_exec time:30];
  }
}

/* ---------------------------------------------------------------- */
#pragma mark "Actions"
/* ---------------------------------------------------------------- */
//34567890123456789012345678901234567890

- (IBAction)action_pref:(id)sender
{
  char title[64],buf[128];
#if (DEBUG > 0)
  fprintf(stderr,"%s(%p): %s\n",PREFUN,sender,[[sender title] UTF8String]);
#endif
  assert([NSThread isMainThread]);

  (void)Ystr2chr([sender title],title,sizeof(title));
  if (strstr(title,"Preferences")) {   // open 'Preferences' panel
    sprintf(buf,"%d",YprefGetInt(DBE_SATLEV,0xffff,use_shared));
    YeditSet(edit_satlev,buf);
    [_window makeKeyAndOrderFront:self];
  } else
  if (strstr(title,"Apply")) {         // close 'Preferences' panel
    YprefPutInt(DBE_SATLEV,[edit_satlev intValue],use_shared);
    [_window orderOut:self];
    [[NSUserDefaults standardUserDefaults] synchronize];
    [qlt_controller reloadPreferences:YES];
  } else {
    assert(0);
  }
}

/* ---------------------------------------------------------------- */

- (IBAction)action_file:(id)sender
{
  char title[64];
#if (DEBUG > 0)
  fprintf(stderr,"%s(%p)\n",PREFUN,sender);
#endif
  assert([NSThread isMainThread]);

  (void)Ystr2chr([sender title],title,sizeof(title));
  
  if (strstr(title,"Logfile")) {       // open logfile in Console
    [[NSWorkspace sharedWorkspace] openFile:main_logger.logfile];
  } else
  if (strstr(title,"App")) {           // show 'Application Support'
    [[NSWorkspace sharedWorkspace]     // folder not used
      openURL:[NSURL fileURLWithPath:@(s_path) isDirectory:YES]];
  } else
  if (strstr(title,"FITS")) {          // load FITS file
      NSURL *url = [NSURL fileURLWithPath:self.dataPath isDirectory:YES];
      NSOpenPanel *panel = [[NSOpenPanel openPanel] retain];
      [panel setAllowedFileTypes:[NSArray arrayWithObject:@"fits"]];
      [panel setAllowsMultipleSelection:NO];
      [panel setDirectoryURL:url];
      [panel beginSheetModalForWindow:[qlt_controller window]
             completionHandler:^(NSInteger returnCode)
      {
        if (returnCode == NSModalResponseOK) {
          NSURL *url = [[panel URLs] objectAtIndex:0]; // NEW moved here
          [self loadFITS:url]; // for easier reuse of the 'loadFITS' method
          self.dataPath = [[url URLByDeletingLastPathComponent] path];
        }
        [panel release];
      }];
  } else {
    assert(0);
  }
}

/* ---------------------------------------------------------------- */

- (IBAction)action_help:(id)sender
{
  char title[64];
#if (DEBUG > 0)
  fprintf(stderr,"%s(%p)\n",PREFUN,sender);
#endif

  Ystr2chr([sender title],title,sizeof(title));
  if (strstr(title,"Software")) {
    Ywww("http://instrumentation.obs.carnegiescience.edu/Software/M2FS");
  } else {
    assert(0);
  }
}

/* ---------------------------------------------------------------- */
#pragma mark "TCPIP_Server delegate"
/* ---------------------------------------------------------------- */

// NEW this method handles the incoming commands

- (char*)tcpip_handler:(const char*)command from:(NSString*)ipNumber socket:(int)msgsock
{
  NSString *string = @(command);
  NSString *line = [string stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceAndNewlineCharacterSet]];
#if (DEBUG > 0)
  fprintf(stderr,"%s(%s,%s,%d)\n",PREFUN,[line UTF8String],[ipNumber UTF8String],msgsock);
#endif

  string = [NSString stringWithFormat:@"TCP: %@-> %@",ipNumber,line];
  [main_logger append:[string UTF8String]];

  char *answer = (char*)malloc(512*sizeof(char));
  *answer = '\0';

  NSArray *a = [line componentsSeparatedByString:@" "];
  NSString *cmd = a[0];
  if ([cmd compare:@"version"] == NSOrderedSame) {
    sprintf(answer,"%s (%s)",p_version,p_build);
  } else
  if ([cmd compare:@"load"] == NSOrderedSame) {
    if ([a count] > 1) {
      NSURL *url = [NSURL fileURLWithPath:a[1] isDirectory:NO];
      [self loadFITS:url];
      sprintf(answer,"%s",[[url lastPathComponent] UTF8String]);
    } else {
      sprintf(answer,"missing file");
    }
  }

  string = [NSString stringWithFormat:@"TCP: %@<- %s",ipNumber,answer];
  [main_logger append:[string UTF8String]];
  strcat(answer,"\n");                 // terminating <LF>
  
  return answer;                       // automatically free()ed by the server
}

/* ---------------------------------------------------------------- */

@end

/* ---------------------------------------------------------------- */
/* ---------------------------------------------------------------- */
/* ---------------------------------------------------------------- */
