/* ---------------------------------------------------------------- */
//
// MagView.m
//
// Created by Christoph Birk on 2012-03-12
// Copyright 2012 Carnegie Observatories. All rights reserved.
//
/* ---------------------------------------------------------------- */

#ifndef DEBUG
#define DEBUG           1              // debug level
#endif

/* ---------------------------------------------------------------- */

#import "MagView.h"
#import "QltoolController.h"
#import "Yutils.h"                     // Cocoa wrappers

/* ---------------------------------------------------------------- */

@implementation MagView

@synthesize controller,safeBounds,radius,mag;

/* ---------------------------------------------------------------- */
//34567890123456789012345678901234567890

- (void)awakeFromNib
{
  u_int *idata,*d;
#if (DEBUG > 1)
  fprintf(stderr,"%s:%p\n",PREFUN,self);
#endif

  safeBounds = [self bounds];
  int w = (int)self.bounds.size.width;
  int h = (int)self.bounds.size.height;
  d = idata = (u_int*)malloc(w*h*sizeof(u_int));
  for (int j=0; j<w*h; j++) *d++ = 0x00f0a0a0; // 0BGR
  YimageData(self,idata,w,h);
  free((void*)idata);
}

/* ---------------------------------------------------------------- */

- (void)drawRect:(NSRect)rect
{
#if (DEBUG > 1) 
  fprintf(stderr,"%s([%.0f,%.0f,%d])\n",PREFUN,rect.size.width,rect.size.height,radius);
#endif
  assert([NSThread isMainThread]);     // NSView is mainThread only

  [super drawRect:rect];               // draw image

  safeBounds = [self bounds];

  if (radius > 0) {                    // draw center pixel and aperture
    if ([self lockFocusIfCanDraw]) { float x,y; CGContextRef context;
      context = [[NSGraphicsContext currentContext] graphicsPort];
      x = rect.size.width/2.0f; y = rect.size.height/2.0f;
      CGContextSetRGBStrokeColor(context,0.0f,1.0f,0.0f,1);
      CGContextSetLineWidth(context,1);
      CGContextAddArc(context,x,y,radius,0,2*3.14159,0);
      CGContextStrokePath(context);
      CGContextStrokeRect(context,CGRectMake(x-mag/2-1,y-mag/2-1,mag+1,mag+1));
      [self unlockFocus];
    }
  }
}

/* ---------------------------------------------------------------- */
/* NSView overrides */
/* ---------------------------------------------------------------- */

- (void)mouseDown:(NSEvent*)event
{
#if (DEBUG > 1)
  fprintf(stderr,"%s(%p)\n",PREFUN,event);
#endif
  assert([NSThread isMainThread]); 

  [[self window] makeFirstResponder:self];
}

/* ---------------------------------------------------------------- */

- (void)keyDown:(NSEvent*)event
{
  int dx=0,dy=0;
  QltoolController *qltc=(QltoolController*)controller;
#if (DEBUG > 1)
  fprintf(stderr,"%s(%p), keyCode=%d\n",PREFUN,event,[event keyCode]);
#endif
  assert(qltc);

  NSString *chars = [event charactersIgnoringModifiers];
  switch ([chars characterAtIndex:0]) {
    case NSLeftArrowFunctionKey:  dx = -1; dy =  0; break;
    case NSRightArrowFunctionKey: dx = +1; dy =  0; break;
    case NSDownArrowFunctionKey:  dx =  0; dy = +1; break;
    case NSUpArrowFunctionKey:    dx =  0; dy = -1; break;
  }
  if ((dx == 0) && (dy == 0)) {
    if ([chars compare:@" "] == NSOrderedSame) {   // FWHM fit
      [qltc update_fwhm:YES];
    }
    return;
  }
  
  NSUInteger mod = [event modifierFlags]; 
  if (mod & NSEventModifierFlagControl)   { dx *= 5; dy *=5; }
  if (mod & NSEventModifierFlagOption)    { dx *= 5; dy *=5; }
  if (mod & NSEventModifierFlagCommand)   { dx *= 5; dy *=5; }
  if (mod & NSEventModifierFlagShift)     { dx *= 5; dy *=5; }
  [qltc incrementMouse:dx dy:dy];
  [qltc update_magnifier];
  
  if (qltc.fwhm > 0) [qltc update_fwhm:YES];
}

/* ---------------------------------------------------------------- */

@end

/* ---------------------------------------------------------------- */
/* ---------------------------------------------------------------- */
/* ---------------------------------------------------------------- */

