/* ---------------------------------------------------------------- */
//
// QltoolController.h
//
// Created by Christoph Birk on 2011-12-08
// Copyright 2011 Carnegie Observatories. All rights reserved.
//
/* ---------------------------------------------------------------- */

#import  "OverView.h"
#import  "MagView.h"

/* ---------------------------------------------------------------- */

#define DBE_SATLEV      "dbe_satlev"
#define DEF_SATLEV      44000

#define QLT_NCOLS       200

enum { LUT_GRAY,LUT_BBDY,LUT_RAIN,LUT_INV };

#define LUT_GREY LUT_GRAY

enum { STRETCH_CUTS,STRETCH_MED3,STRETCH_MED5,STRETCH_310,STRETCH_MIMA };

/* ---------------------------------------------------------------- */

@interface QltoolController : NSWindowController {
  int      flip_x,flip_y;
  int      raw_w,raw_h;
  int      lut,stretch,radius;
  double   slevel,contrast;
  int      data_min,data_max,data_dyn;
  int      *raw_data;
  u_int    *pixels;
  int      satlev;
  // Overview
  IBOutlet NSView *root_view;          // parent of over_view
  IBOutlet NSTextField *edit_min,*edit_max;
  IBOutlet NSPopUpButton *popup_lut,*popup_stretch;
  OverView *over_view; 
  // Magnifier
  IBOutlet MagView *mag_view;
  IBOutlet NSTextField *edit_pixel,*edit_minmax,*edit_avesig,*edit_radius,*edit_fwhm;
  IBOutlet NSStepper *step_radius;
  IBOutlet NSPopUpButton *popup_mag;
  int      mag,mag_x,mag_y;
  float    fwhm,flux;
}

@property (readonly)           int    raw_w,raw_h;

@property (nonatomic)          double contrast,slevel;
@property (nonatomic)          float  fwhm;
@property (readonly,nonatomic) u_int  *pixels;
@property (readonly,nonatomic) int    satlev,lut;

@property (assign,nonatomic,readonly) MagView *mag_view;
@property (readonly,nonatomic)        int mag,mag_x,mag_y;

/* ---------------------------------------------------------------- */

- (id)initWithWindowNib:(NSString*)name;
- (void)reloadPreferences:(BOOL)redraw;
- (void)fill_pixels;

- (IBAction)action_lut:(id)sender;
- (IBAction)action_stretch:(id)sender;
- (IBAction)action_minmax:(id)sender;
- (IBAction)action_mag:(id)sender;
- (IBAction)action_radius:(id)sender;

- (void)reset:(BOOL)clear width:(int)w height:(int)h title:(NSString*)tstr;
- (void)updateData:(int*)ptr line:(int)y;
- (void)redrawWithScaling:(BOOL)scale;
- (void)redrawPixels;
- (void)update_magnifier;
- (void)update_fwhm:(BOOL)recalc;

- (void)setMouse:(NSPoint)point;
- (void)incrementMouse:(int)dx dy:(int)dy;

/* ---------------------------------------------------------------- */

@end

/* ---------------------------------------------------------------- */
/* ---------------------------------------------------------------- */
/* ---------------------------------------------------------------- */
